@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              Card.h
 //------------------------------------------------------------------------------
-// $Id: Card.h,v 1.10 2006/09/06 02:07:59 vlg Exp $
+// $Id: Card.h,v 1.11 2006/12/18 03:33:14 vlg Exp $
 //
 // Date: Jan 8, 2004
 //------------------------------------------------------------------------------
@@ -20,14 +20,14 @@ public:
 	virtual void set_question     (const ustring& q_);
 	virtual void set_answer       (const ustring& a_);
 	virtual void set_example      (const ustring& e_);
-	virtual void set_alt_spelling (const ustring& a_);
+	virtual void set_alt_spelling (SideSelection side_, const ustring& a_);
 
 	void set_id (long id_);
 
 	virtual ustring get_question     () const { return m_question;     }
 	virtual ustring get_answer       () const { return m_answer;       }
 	virtual ustring get_example      () const { return m_example;      }
-	virtual ustring get_alt_spelling () const { return m_alt_spelling; }
+	virtual ustring get_alt_spelling (SideSelection side_) const;
 	virtual long    get_id           () const { return m_id;           }
 
 	virtual void dump () const;
@@ -37,13 +37,53 @@ protected:
 	virtual bool equal_expiration     (const VCard& rhs_) { return true; }
 
 private:
-	long    m_id;
-	ustring m_question;
-	ustring m_answer;
-	ustring m_example;
-	ustring m_alt_spelling;
+	long      m_id;
+	ustring   m_question;
+	ustring   m_answer;
+	ustring   m_example;
+	ustring   m_front_alt_spelling;
+	ustring   m_back_alt_spelling;
 };
 
+inline ustring
+Card::get_alt_spelling (SideSelection side_) const 
+{ 
+	trace_with_mask("Card::get_alt_spelling",GUITRACE);
+
+	ustring ret;
+	if (side_ == FRONT) {
+		DL ((GRAPP,"Return front.alt.spell = %s\n",
+			 m_front_alt_spelling.c_str ()));
+		ret = m_front_alt_spelling;
+	}
+	else {
+		DL ((GRAPP,"Return back.alt.spell = %s\n",
+			 m_back_alt_spelling.c_str ()));
+		ret = m_back_alt_spelling;
+	}
+	return ret;
+		
+#ifdef BLOCK
+	return (side_ == FRONT ? m_front_alt_spelling : m_back_alt_spelling);
+#endif
+}
+
+inline void
+Card::set_alt_spelling (SideSelection side_, const ustring& a_) 
+{ 
+//	trace_with_mask("Card::set_alt_spelling",GUITRACE);
+
+	if (side_ == FRONT) {
+		DL ((GRAPP,"Set front.alt.spell = %s\n", a_.c_str ()));
+		m_front_alt_spelling = a_;
+	}
+	else {
+		DL ((GRAPP,"Set back.alt.spell = %s\n", a_.c_str ()));
+		m_back_alt_spelling = a_;
+	}
+	mark_dirty (); 
+}
+
 inline void 
 Card::set_question (const ustring& q_) 
 { 
@@ -65,13 +105,6 @@ Card::set_example (const ustring& e_)
 	mark_dirty (); 
 }
 
-inline void
-Card::set_alt_spelling (const ustring& a_) 
-{ 
-	m_alt_spelling = a_;
-	mark_dirty (); 
-}
-
 inline void
 Card::set_id (long id_)
 { 
