@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: CardBox.cpp,v 1.58 2006/10/06 02:04:47 vlg Exp $
+// $Id: CardBox.cpp,v 1.64 2007/01/14 02:00:12 vlg Exp $
 //------------------------------------------------------------------------------
 //                            CardBox.cpp
 //------------------------------------------------------------------------------
@@ -241,6 +241,12 @@ find_pixmap_path (const char* pixmap_)
  * CardBox class methods
  *------------------------------------------------------------------------------
  */
+#if defined(IS_PDA) && (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION >= 6)
+#  define CB_SHRINK_ICONS ,34,39,false
+#else
+#  define CB_SHRINK_ICONS
+#endif
+
 CardBox::
 CardBox ()
 {
@@ -253,18 +259,12 @@ CardBox ()
 		m_cdplayers [i] = NULL;
 	}
 
-	m_drawer_empty = 
-		Gdk::Pixbuf::create_from_file(find_pixmap_path ("drawer_empty.png"));
-
-	DL ((GRAPP,"Success on first icon\n"));
-
-	m_drawer_one   = 
-		Gdk::Pixbuf::create_from_file(find_pixmap_path ("drawer_one.png"));
-	DL ((GRAPP,"second icon ok\n"));
-
-	m_drawer_many  = 
-		Gdk::Pixbuf::create_from_file(find_pixmap_path ("drawer_many.png"));
-	DL ((GRAPP,"third icon ok\n"));
+	m_drawer_empty = Gdk::Pixbuf::create_from_file(
+		find_pixmap_path ("drawer_empty.png") CB_SHRINK_ICONS);
+	m_drawer_one   = Gdk::Pixbuf::create_from_file(
+		find_pixmap_path ("drawer_one.png")   CB_SHRINK_ICONS);
+	m_drawer_many  = Gdk::Pixbuf::create_from_file(
+		find_pixmap_path ("drawer_many.png")  CB_SHRINK_ICONS);
 
 	m_sched_green  = 
 		Gdk::Pixbuf::create_from_file(find_pixmap_path ("sched_green.png"));
@@ -275,8 +275,6 @@ CardBox ()
 	m_sched_yellow = 
 		Gdk::Pixbuf::create_from_file(find_pixmap_path ("sched_yellow.png"));
 
-	DL ((GRAPP,"last icon ok\n"));
-
 	for (i = 0; i < CARD_BOX_SIZE; i++) {
 		m_cbs.push_back (CardBoxIF ());
 		m_cbs [i].m_sched_light = manage (new Gtk::Image);
@@ -309,7 +307,11 @@ CardBox ()
 		m_cbs [i].m_total_count->set_padding (8,0);
 		m_cbs [i].m_total_count->set_justify (Gtk::JUSTIFY_RIGHT);
 		m_cbs [i].m_total_count->set_line_wrap (false);
+#if defined (IS_PDA)
+		m_cbs [i].m_total_count->set_text (_("0"));
+#else
 		m_cbs [i].m_total_count->set_text (_("0 cards"));
+#endif
 	}
 
 	Gtk::Table* cbt = manage (new Gtk::Table (5, 3, false));
@@ -359,11 +361,16 @@ CardBox::
 update_count (int idx_)
 {
 	std::ostringstream os;
-	
-	os << m_card_deck [idx_].size () << " cards";
-	m_cbs [idx_].m_total_count->set_text (os.str ());
 
+#ifdef IS_PDA
+	os << m_card_deck [idx_].size ();
+#else
+	os << m_card_deck [idx_].size () << " cards";
+#endif
+
+	m_cbs [idx_].m_total_count->set_text (os.str ());
 	m_cbs [idx_].m_box_pic->clear ();	
+
 	if (m_card_deck [idx_].size () == 0) {
 		m_cbs [idx_].m_box_pic->set (m_drawer_empty);
 	}
