@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: CardBox.h,v 1.29 2006/09/30 20:42:27 vlg Exp $
+// $Id: CardBox.h,v 1.30 2006/12/02 05:43:10 vlg Exp $
 //------------------------------------------------------------------------------
 //                            CardBox.h
 //------------------------------------------------------------------------------
@@ -104,12 +104,7 @@ private:
 					   m_total_count (NULL), m_expired_count (0) 
 			{ /* no-op */ }
 
-		void reset () 
-		{
-			m_sched_count->set_text ("0");
-			m_total_count->set_text ("0 cards");
-			m_expired_count = 0;
-		}
+		void reset ();
 		void add_one () { m_expired_count++; }
 	};
 
@@ -166,4 +161,19 @@ inline CardBox::
 	trace_with_mask("CardBox::~CardBox",GUITRACE);
 }
 
+inline void
+CardBox::CardBoxIF::
+reset () 
+{
+	m_sched_count->set_text ("0");
+	m_expired_count = 0;
+
+#ifdef IS_PDA
+	m_total_count->set_text ("0");
+#else
+	m_total_count->set_text ("0 cards");
+#endif
+
+}
+
 #endif /* CARD_BOX_H */
