@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              CardDeck.cpp
 //------------------------------------------------------------------------------
-// $Id: CardDeck.cpp,v 1.25 2006/09/30 20:42:27 vlg Exp $
+// $Id: CardDeck.cpp,v 1.26 2006/12/18 03:33:14 vlg Exp $
 //
 //  Copyright (c) 2004,2006 by Vladislav Grinchenko
 //
@@ -75,11 +75,11 @@ erase (VCard* vcard_)
 
 void 
 CardDeck::
-play_card (VDeck::cardlist_iterator& iter_)
+play_card (SideSelection side_, VDeck::cardlist_iterator& iter_)
 {
 	CardRef* cref = dynamic_cast<CardRef*>(*iter_);
 	Assure_exit (cref);
-	cref->get_deck ().play_card (iter_);
+	cref->get_deck ().play_card (side_, iter_);
 }
 
 void 
