@@ -2,9 +2,9 @@
 //------------------------------------------------------------------------------
 //                              CardView.cpp
 //------------------------------------------------------------------------------
-// $Id: CardView.cpp,v 1.37 2006/09/30 20:42:27 vlg Exp $
+// $Id: CardView.cpp,v 1.50 2007/01/14 02:00:13 vlg Exp $
 //
-//  Copyright (c) 2004-2005 by Vladislav Grinchenko
+//  Copyright (c) 2004-2006 by Vladislav Grinchenko
 //
 //  This program is free software; you can redistribute it and/or 
 //  modify it under the terms of the GNU General Public License   
@@ -14,6 +14,7 @@
 // Date: Feb 15, 2004
 //------------------------------------------------------------------------------
 
+#include <gtkmmconfig.h>
 #include <gtkmm/image.h>
 #include <gtkmm/label.h>
 #include <gtkmm/box.h>
@@ -23,6 +24,7 @@
 #include <gtkmm/scrolledwindow.h>
 #include <gtkmm/table.h>
 #include <gtkmm/messagedialog.h>
+#include <gtkmm/paned.h>
 
 #include "Granule.h"
 #include "MainWindow.h"
@@ -32,7 +34,7 @@
 #include "Card.h"
 #include "CardRef.h"
 #include "EditControls.h"
-#include "Intern.h"
+#include "Intern.h"				/* always last */
 
 #define SET_CV_LABEL(l) \
 	l->set_alignment  (0.5,0.5);  \
@@ -57,7 +59,17 @@
 	t->set_justification (Gtk::JUSTIFY_LEFT);  \
 	t->set_accepts_tab (false);
 
+#define SET_CV_SCROLLWIN(w,t) \
+	w->set_flags (Gtk::CAN_FOCUS); \
+	w->set_shadow_type (Gtk::SHADOW_NONE); \
+	w->set_policy (Gtk::POLICY_ALWAYS, Gtk::POLICY_ALWAYS); \
+	w->property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT); \
+	w->add (*t)
 
+/*------------------------------------------------------------------------------
+** CardView class implementation
+**------------------------------------------------------------------------------
+*/
 CardView::
 CardView (VCard* card_) 
 	: 
@@ -71,6 +83,7 @@ CardView (VCard* card_)
 	m_keep_open     (false),
 
 	m_cancel_button (NULL),
+	m_count_button  (NULL),
 	m_add_n_close_button (NULL),
 	m_add_button    (NULL),
 	m_help_button   (NULL),
@@ -78,7 +91,8 @@ CardView (VCard* card_)
 	m_front_text    (NULL),
 	m_back_text     (NULL),
 	m_example_text  (NULL),
-	m_altspell_text (NULL),
+	m_asf_text      (NULL),
+	m_asb_text      (NULL),
 
 	m_edit_controls (NULL),
 	m_help_dialog   (*this)
@@ -107,52 +121,72 @@ CardView (VCard* card_)
 	Gtk::Alignment* alignment2;
 	Gtk::Alignment* alignment3;
 	Gtk::Alignment* alignment4;
-	Gtk::Alignment* alignment5;
-	Gtk::Alignment* alignment6;
-	Gtk::Alignment* alignment7;
-	Gtk::Alignment* alignment8;
-	Gtk::Alignment* alignment9;
+
+	Gtk::Alignment* bt_alignment;
+	Gtk::Alignment* ft_alignment;
+	Gtk::Alignment* et_alignment;
+
+	Gtk::Alignment* asf_alignment;
+	Gtk::Alignment* asb_alignment;
 
 	Gtk::ScrolledWindow* front_scrollwin;
 	Gtk::ScrolledWindow* backtext_scrollwin;
 	Gtk::ScrolledWindow* example_scrollwin;
-	Gtk::ScrolledWindow* altspell_scrollwin;
+	Gtk::ScrolledWindow* asf_scrollwin;
+	Gtk::ScrolledWindow* asb_scrollwin;
 
 	Gtk::Label* back_label;
 	Gtk::Label* example_label;
 	Gtk::Label* front_label;
-	Gtk::Label* altspell_label;
+	Gtk::Label* asf_label;
+	Gtk::Label* asb_label;
 
 	Gtk::VBox* vbox_frontlabel;
 	Gtk::VBox* vbox_backlabel;
 	Gtk::VBox* vbox_examplelabel;
-	Gtk::VBox* vbox_altspelllabel;
+	Gtk::HBox* hbox_asf;
+	Gtk::HBox* hbox_asb;
+
+	Gtk::Table* elems_table_1;
+	Gtk::Table* elems_table_2;
+	Gtk::Table* elems_table_3;
+
+	Gtk::VPaned* paned_1;		/* Holds elems_table_1 and paned_2       */
+	Gtk::VPaned* paned_2;		/* Holds elems_table_2 and elems_table_3 */
 
-	Gtk::Table* elems_table;
 	Gtk::VBox*  vbox_all;
 
-#if defined (IS_HILDON)
+	set_title ("Card View");
+
+#ifdef IS_HILDON
 	set_size_request (672, 396);
 #else
-	set_icon_from_file (GRAPPDATDIR "/pixmaps/cardview_32x32.png");
+	set_modal (true);  // very important - we're not protected from multi-clicks
+//	property_window_position ().set_value (Gtk::WIN_POS_CENTER);
 
+#ifdef IS_PDA
+	set_resizable (false);
+	Gdk::Geometry dp_geometry =	{ 240, 320,	240, 320, 240, 320,-1, -1, -1, -1};
+	set_geometry_hints (*this, dp_geometry, 
+				Gdk::HINT_MIN_SIZE | Gdk::HINT_MAX_SIZE | Gdk::HINT_BASE_SIZE);
+#else
+	set_resizable (true);
+	set_icon_from_file (GRAPPDATDIR "/pixmaps/cardview_32x32.png");
 	Gdk::Rectangle geom = CONFIG->get_card_view_geometry ();
 	set_size_request (geom.get_width (), geom.get_height ());
-
-	set_title ("Card View");
-	set_resizable (true);
-	set_modal (true);  // very important - we're not protected from multi-clicks
 	set_transient_for (*MAINWIN);
-	property_window_position ().set_value (Gtk::WIN_POS_CENTER);
-#endif	// ifdef IS_HILDON
+#endif // (!IS_PDA)
 
-	/** <Clear> button
+#endif	// (!IS_HILDON)
+
+	/** <Clear> button - OBSOLETE -
 	 */
-	image1 = Gtk::manage (new Gtk::Image (Gtk::StockID ("gtk-clear"), 
+	image1 = Gtk::manage (new Gtk::Image (Gtk::StockID ("gtk-help"), 
 										  Gtk::IconSize (4)));
-	label1     = Gtk::manage (new Gtk::Label (_("Clear")));
-	hbox1      = Gtk::manage (new Gtk::HBox (false, 2));
-	alignment1 = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
+	label1        = Gtk::manage (new Gtk::Label (_("Help")));
+	hbox1         = Gtk::manage (new Gtk::HBox (false, 2));
+	alignment1    = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
+	m_help_button = Gtk::manage (new Gtk::Button ());
 
 	/** <Cancel> Button
 	 */
@@ -163,13 +197,17 @@ CardView (VCard* card_)
 	alignment2      = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
 	m_cancel_button = Gtk::manage (new Gtk::Button ());
 
+	/** <Count> *Button*
+	 */
+	m_count_button  = Gtk::manage (new Gtk::Button ());
+
 	/** <Add> Button
 	 */
-	image3      = Gtk::manage (new Gtk::Image (Gtk::StockID ("gtk-add"), 
-												   Gtk::IconSize (4)));
-	label3      = Gtk::manage (new Gtk::Label (_("Add")));
-	hbox3       = Gtk::manage (new Gtk::HBox (false, 2));
-	alignment3  = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
+	image3       = Gtk::manage (new Gtk::Image (Gtk::StockID ("gtk-add"), 
+												Gtk::IconSize (4)));
+	label3       = Gtk::manage (new Gtk::Label (_("Add")));
+	hbox3        = Gtk::manage (new Gtk::HBox (false, 2));
+	alignment3   = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
 	m_add_button = Gtk::manage (new Gtk::Button ());
 
 	/** <Add'n'Close> Button
@@ -181,79 +219,119 @@ CardView (VCard* card_)
 	alignment4  = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
 	m_add_n_close_button = Gtk::manage (new Gtk::Button ());
 
-	/** <Help> Button
-	 */
-	image5      = Gtk::manage (new Gtk::Image (Gtk::StockID ("gtk-help"), 
-												   Gtk::IconSize (4)));
-	label5      = Gtk::manage (new Gtk::Label (_("Help")));
-	hbox5       = Gtk::manage (new Gtk::HBox (false, 2));
-	alignment8  = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
-	m_help_button = Gtk::manage (new Gtk::Button ());
-
 	/** Text
 	 */
 	m_front_text       = Gtk::manage (new Gtk::TextView);
 	front_scrollwin    = Gtk::manage (new Gtk::ScrolledWindow);
-	front_label        = Gtk::manage (new Gtk::Label (_("Front :\n \n \n")));
-	alignment6         = Gtk::manage (new Gtk::Alignment (0.84, 0.5, 0, 0));
+	front_label        = Gtk::manage (new Gtk::Label (_("Front")));
+	ft_alignment       = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
 	vbox_frontlabel    = Gtk::manage (new Gtk::VBox (false, 0));
 
 	m_back_text        = Gtk::manage (new Gtk::TextView);
 	backtext_scrollwin = Gtk::manage (new Gtk::ScrolledWindow);
-	back_label         = Gtk::manage (new Gtk::Label (_("Back :\n \n \n")));
-	alignment5         = Gtk::manage (new Gtk::Alignment (0.76, 0.5, 0, 0));
+	back_label         = Gtk::manage (new Gtk::Label (_("Back")));
+	bt_alignment       = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
 	vbox_backlabel     = Gtk::manage (new Gtk::VBox (false, 0));
 
 	m_example_text     = Gtk::manage (new Gtk::TextView);
 	example_scrollwin  = Gtk::manage (new Gtk::ScrolledWindow);
-	example_label      = Gtk::manage (new Gtk::Label (_("Example : ")));
-	alignment7         = Gtk::manage (new Gtk::Alignment (0.84, 0.5, 0, 0));
+	example_label      = Gtk::manage (new Gtk::Label (_("Example")));
+	et_alignment       = Gtk::manage (new Gtk::Alignment (0.5, 0.5, 0, 0));
 	vbox_examplelabel  = Gtk::manage (new Gtk::VBox (false, 0));
 
-	m_altspell_text     = Gtk::manage (new Gtk::TextView);
-	altspell_scrollwin  = Gtk::manage (new Gtk::ScrolledWindow);
-	altspell_label      = Gtk::manage (new Gtk::Label (_("Alt\nSpelling : ")));
-	alignment9          = Gtk::manage (new Gtk::Alignment (0.84, 0.5, 0, 0));
-	vbox_altspelllabel  = Gtk::manage (new Gtk::VBox (false, 0));
+	m_asf_text     = Gtk::manage (new Gtk::TextView);
+	asf_scrollwin  = Gtk::manage (new Gtk::ScrolledWindow);
+	asf_label      = Gtk::manage (new Gtk::Label (_("ASF")));
+	asf_alignment  = Gtk::manage (new Gtk::Alignment (0, 0.5, 0, 0));
+	hbox_asf       = Gtk::manage (new Gtk::HBox (false, 0));
 
-	elems_table = Gtk::manage (new Gtk::Table (2, 5, false));
-	vbox_all = Gtk::manage (new Gtk::VBox (false, 0));
+	m_asb_text     = Gtk::manage (new Gtk::TextView);
+	asb_scrollwin  = Gtk::manage (new Gtk::ScrolledWindow);
+	asb_label      = Gtk::manage (new Gtk::Label (_("ASB")));
+	asb_alignment  = Gtk::manage (new Gtk::Alignment (0, 0.5, 0, 0));
+	hbox_asb       = Gtk::manage (new Gtk::HBox (false, 0));
+
+	elems_table_1  = Gtk::manage (new Gtk::Table (2, 2, false));
+	elems_table_2  = Gtk::manage (new Gtk::Table (2, 1, false));
+	elems_table_3  = Gtk::manage (new Gtk::Table (2, 2, false));
+
+	paned_1        = Gtk::manage (new Gtk::VPaned);
+	paned_2        = Gtk::manage (new Gtk::VPaned);
+
+	vbox_all       = Gtk::manage (new Gtk::VBox (false, 0));
 
 	m_front_text   ->modify_font (CONFIG->get_example_font ());
 	m_back_text    ->modify_font (CONFIG->get_example_font ());
 	m_example_text ->modify_font (CONFIG->get_example_font ());
-	m_altspell_text->modify_font (CONFIG->get_example_font ());
+	m_asf_text     ->modify_font (CONFIG->get_example_font ());
+	m_asb_text     ->modify_font (CONFIG->get_example_font ());
 
+
+	/**----------------**
+	 **  Edit Controls **
+	 **----------------**
+	 */
+	m_edit_controls = Gtk::manage(new EditControls(m_front_text->get_buffer()));
+	m_edit_controls->unset_flags (Gtk::CAN_FOCUS);
+	vbox_all->pack_start (*m_edit_controls, Gtk::PACK_SHRINK, 0);
+
+	/**---------------**
+	 ** Setup buttons **
+	 **---------------**
+	 */
+	
+	/** [Help] Button
+	 */
 	image1->set_alignment  (0.5,0.5);
 	image1->set_padding    (0,0);
-	SET_CV_LABEL(label1);
 
+#if !defined(IS_PDA)
+	SET_CV_LABEL(label1);
 	hbox1->pack_start (*image1, Gtk::PACK_SHRINK, 0);
 	hbox1->pack_start (*label1, Gtk::PACK_SHRINK, 0);
 	alignment1->add (*hbox1);
+	m_help_button->set_relief  (Gtk::RELIEF_NORMAL);
+	m_help_button->add (*alignment1);
+#else
+	m_help_button->add (*image1);
+#endif
+	m_help_button->unset_flags (Gtk::CAN_FOCUS); // Take help off the chain
 
+	/** [Cancel] Button
+	 */
 	image2->set_alignment  (0.5,0.5);
 	image2->set_padding    (0,0);
+
 	SET_CV_LABEL(label2);
 	hbox2->pack_start (*image2, Gtk::PACK_SHRINK, 0);
 	hbox2->pack_start (*label2, Gtk::PACK_SHRINK, 0);
 	alignment2->add (*hbox2);
-
-	m_cancel_button->set_flags (Gtk::CAN_FOCUS);
+	m_cancel_button->unset_flags (Gtk::CAN_FOCUS);
 	m_cancel_button->set_relief (Gtk::RELIEF_NORMAL);
 	m_cancel_button->add (*alignment2);
 
+	/** [Count] Label
+	 */
+	m_count_button->set_focus_on_click (false);
+	m_count_button->unset_flags (Gtk::CAN_FOCUS);
+	m_count_button->set_relief (Gtk::RELIEF_NONE);
+	m_count_button->set_use_stock (false);
+	m_count_button->set_use_underline (false);
+
+	/** [+ Add] Button
+	 */
 	image3->set_alignment  (0.5,0.5);
 	image3->set_padding    (0,0);
 	SET_CV_LABEL(label3);
 	hbox3->pack_start (*image3, Gtk::PACK_SHRINK, 0);
 	hbox3->pack_start (*label3, Gtk::PACK_SHRINK, 0);
 	alignment3->add (*hbox3);
-
 	m_add_button->set_flags (Gtk::CAN_FOCUS);
 	m_add_button->set_relief (Gtk::RELIEF_NORMAL);
 	m_add_button->add (*alignment3);
 
+	/** [Add-n-Close] 
+	 */
 	image4->set_alignment  (0.5,0.5);
 	image4->set_padding    (0,0);
 	SET_CV_LABEL(label4);
@@ -265,95 +343,78 @@ CardView (VCard* card_)
 	m_add_n_close_button->set_relief (Gtk::RELIEF_NORMAL);
 	m_add_n_close_button->add (*alignment4);
 
-	image5->set_alignment  (0.5,0.5);
-	image5->set_padding    (0,0);
-	SET_CV_LABEL(label5);
-	hbox5->pack_start (*image5, Gtk::PACK_SHRINK, 0);
-	hbox5->pack_start (*label5, Gtk::PACK_SHRINK, 0);
-	alignment8->add (*hbox5);
-
-	m_help_button->unset_flags (Gtk::CAN_FOCUS); // Take help off the chain
-	m_help_button->set_relief (Gtk::RELIEF_NORMAL);
-	m_help_button->add (*alignment8);
-
 	/**--------------**
 	 **  Front Text  **
 	 **--------------**
 	 */
 	SET_CV_TEXT(m_front_text);
-	m_front_text->get_buffer ()->set_text (_("Fill up the front of the card"));
-
-	front_scrollwin->set_flags (Gtk::CAN_FOCUS);
-	front_scrollwin->set_shadow_type (Gtk::SHADOW_NONE);
-	front_scrollwin->set_policy (Gtk::POLICY_ALWAYS, Gtk::POLICY_ALWAYS);
-	front_scrollwin->
-		property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
-	front_scrollwin->add (*m_front_text);
-
+	SET_CV_SCROLLWIN(front_scrollwin,m_front_text);
 	SET_CV_LABEL(front_label);
-	alignment6->add (*front_label);
-	vbox_frontlabel->pack_start (*alignment6, Gtk::PACK_SHRINK, 0);
+#ifndef IS_HILDON
+#if (GTKMM_MINOR_VERSION < 6)
+	front_label->set_text ("Q:");
+#else
+	front_label->set_angle (90.0);
+#endif
+#endif
+	ft_alignment->add (*front_label);
+	vbox_frontlabel->pack_start (*ft_alignment, Gtk::PACK_SHRINK, 0);
 	
-
 	/**--------------**
 	 **   Back Text  **
 	 **--------------**
 	 */
 	SET_CV_TEXT(m_back_text);
-	m_back_text->get_buffer ()->set_text (_("Fill up the back of the card"));
-
-	backtext_scrollwin->set_flags (Gtk::CAN_FOCUS);
-	backtext_scrollwin->set_shadow_type (Gtk::SHADOW_NONE);
-	backtext_scrollwin->set_policy (Gtk::POLICY_ALWAYS, Gtk::POLICY_ALWAYS);
-	backtext_scrollwin->
-		property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
-	backtext_scrollwin->add (*m_back_text);
-
+	SET_CV_SCROLLWIN(backtext_scrollwin,m_back_text);
 	SET_CV_LABEL(back_label);
-	alignment5->add (*back_label);
-	vbox_backlabel->pack_start (*alignment5, Gtk::PACK_SHRINK, 0);
+#ifndef IS_HILDON
+#if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION >= 6)
+	back_label->set_angle (90.0);
+#else
+	back_label->set_text ("A:");
+#endif
+#endif
+	bt_alignment->add (*back_label);
+	vbox_backlabel->pack_start (*bt_alignment, Gtk::PACK_SHRINK, 0);
 
 	/**----------------**
 	 **  Example Text  **
 	 **----------------**
 	 */
 	SET_CV_TEXT(m_example_text);
-	m_example_text->get_buffer ()->set_text (_("Fill up an example"));
-
-	example_scrollwin->set_flags (Gtk::CAN_FOCUS);
-	example_scrollwin->set_shadow_type (Gtk::SHADOW_NONE);
-	example_scrollwin->set_policy (Gtk::POLICY_ALWAYS, Gtk::POLICY_ALWAYS);
-	example_scrollwin->
-		property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
-	example_scrollwin->add (*m_example_text);
-
+	SET_CV_SCROLLWIN(example_scrollwin,m_example_text);
 	SET_CV_LABEL(example_label);
+#ifndef IS_HILDON
+#if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION >= 6)
+	example_label->set_angle (90.0);
+#else
+	example_label->set_text ("E:");
+#endif
+#endif
 	vbox_examplelabel->pack_start (*example_label, Gtk::PACK_SHRINK, 0);
 
-	/**-----------------**
-	 **  AltSpell Text  **
-	 **-----------------**
+	/**------------------------------------**
+	 **  AltSpell Text for Front and Back  **
+	 **------------------------------------**
 	 */
-	SET_CV_TEXT(m_altspell_text);
+	SET_CV_TEXT(m_asf_text);
+	SET_CV_LABEL(asf_label);
+	asf_alignment->add (*asf_label);
 
-	altspell_scrollwin->set_flags (Gtk::CAN_FOCUS);
-	altspell_scrollwin->set_shadow_type (Gtk::SHADOW_NONE);
-	altspell_scrollwin->set_policy (Gtk::POLICY_ALWAYS, Gtk::POLICY_ALWAYS);
-	altspell_scrollwin->
-		property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
-	altspell_scrollwin->add (*m_altspell_text);
+	hbox_asf->pack_start (*asf_alignment, Gtk::PACK_SHRINK, 0);
+	hbox_asf->pack_start (*m_asf_text, Gtk::PACK_EXPAND_WIDGET, 0);
+	hbox_asf->unset_flags (Gtk::CAN_FOCUS);
 
-	SET_CV_LABEL(altspell_label);
-	alignment9->add (*altspell_label);
-	vbox_altspelllabel->pack_start (*alignment9, Gtk::PACK_SHRINK, 0);
+	SET_CV_TEXT(m_asb_text);
+	SET_CV_LABEL(asb_label);
+	asb_alignment->add (*asb_label);
 
-	/**----------------**
-	 **  Edit Controls **
-	 **----------------**
+	hbox_asb->pack_start (*asb_alignment, Gtk::PACK_SHRINK, 0);
+	hbox_asb->pack_start (*m_asb_text, Gtk::PACK_EXPAND_WIDGET, 0);
+	hbox_asb->unset_flags (Gtk::CAN_FOCUS);
+
+	/** Hook up callbacks
 	 */
-	m_edit_controls = Gtk::manage(new EditControls(m_front_text->get_buffer()));
-	vbox_examplelabel->pack_end (*m_edit_controls, Gtk::PACK_SHRINK, 0);
-	
 	m_front_text->signal_grab_focus ().connect (
 		bind<Gtk::TextView*> (mem_fun (*m_edit_controls, 
 									   &EditControls::on_focus_changed),
@@ -369,11 +430,14 @@ CardView (VCard* card_)
 									   &EditControls::on_focus_changed),
 							  m_example_text));
 
-	m_altspell_text->signal_grab_focus ().connect (
+	m_asf_text->signal_grab_focus ().connect (
 		bind<Gtk::TextView*> (mem_fun (*m_edit_controls, 
 									   &EditControls::on_focus_changed),
-							  m_altspell_text));
-
+							  m_asf_text));
+	m_asb_text->signal_grab_focus ().connect (
+		bind<Gtk::TextView*> (mem_fun (*m_edit_controls, 
+									   &EditControls::on_focus_changed),
+							  m_asb_text));
 #ifdef IS_HILDON
 	/** Trap 'on virtual keyboard close' event to restore
 	 *  original size of the dialog
@@ -382,87 +446,130 @@ CardView (VCard* card_)
 		mem_fun (*this, &CardView::hildon_im_close_cb));
 #endif
 
-	/**
+	/*************************************************
 	 *  Table holds elements together:
 	 *
-	 *  |----------------+------------------|
-	 *  | vbox(label)    |  scrollwin(text) |
-	 *  |----------------+------------------|
+	 *    0                1                  2
+	 *  0 .----------------+------------------.
+	 *    | vbox(label)    |  scrollwin(text) | Front
+	 *  1 `----------------+------------------'
+	 *    --------------------------[=]-------- <-- pane
+     *    0                1                  2
+	 *  0 .----------------+------------------.
+	 *    |                |  scrollwin(text) | Front Alt Spell
+	 *  1 `----------------+------------------'
+	 *    | vbox(label)    |  scrollwin(text) | Back 
+	 *  2 `----------------+------------------'
+	 *    --------------------------[=]-------- <-- paen
+	 *    0                1                  2
+	 *  0 .----------------+------------------.
+	 *    | vbox(label)    |  scrollwin(text) | Example
+	 *  1 `----------------+------------------'
+	 *    |                |  scrollwin(text) | Back Alt Spell
+	 *  2 `----------------+------------------'
+	 *
+	 *************************************************
 	 */
-	elems_table->set_border_width (4);
-	elems_table->set_row_spacings (8);
-	elems_table->set_col_spacings (2);
-	elems_table->set_name ("CardViewElemsTable");
+
+	elems_table_1->set_border_width (4);
+	elems_table_1->set_row_spacings (2);
+	elems_table_1->set_col_spacings (2);
+	elems_table_1->set_name ("CVElemsTableFront");
+	
+	elems_table_2->set_border_width (4);
+	elems_table_2->set_row_spacings (2);
+	elems_table_2->set_col_spacings (2);
+	elems_table_2->set_name ("CVElemsTableBack");
+	
+	elems_table_3->set_border_width (4);
+	elems_table_3->set_row_spacings (2);
+	elems_table_3->set_col_spacings (2);
+	elems_table_3->set_name ("CVElemsTableExample");
 	
 	/*-----------------------------------------------------------*/
-	elems_table->attach (*vbox_frontlabel, 0, 1, 0, 1, 
-						Gtk::FILL, Gtk::FILL, 0, 0);
+	/*                               y-beg, y-end, x-beg, x-end  */
+	/*-----------------------------------------------------------*/
+	elems_table_1->attach (*vbox_frontlabel, 0, 1, 0, 1, 
+						   Gtk::FILL, Gtk::FILL, 0, 0);
 
-	elems_table->attach (*front_scrollwin, 1, 2, 0, 1, 
-						Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0);
+	elems_table_1->attach (*front_scrollwin, 1, 2, 0, 1, 
+						   Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
 
 	/*-----------------------------------------------------------*/
-	elems_table->attach (*vbox_backlabel, 0, 1, 1, 2, 
-						Gtk::FILL, Gtk::FILL, 0, 0);
-
-	elems_table->attach (*backtext_scrollwin, 1, 2, 1, 2, 
-						Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0);
+	elems_table_2->attach (*hbox_asf, 1, 2, 0, 1, 
+						   Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 0, 0);
 
 	/*-----------------------------------------------------------*/
-	elems_table->attach (*vbox_examplelabel, 0, 1, 2, 3, 
-						Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
+	elems_table_2->attach (*vbox_backlabel, 0, 1, 1, 2,
+						   Gtk::FILL, Gtk::FILL, 0, 0);
 
-	elems_table->attach (*example_scrollwin, 1, 2, 2, 3, 
-						Gtk::FILL, Gtk::FILL, 0, 0);
+	elems_table_2->attach (*backtext_scrollwin, 1, 2, 1, 2, 
+						   Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
 
 	/*-----------------------------------------------------------*/
-	elems_table->attach (*vbox_altspelllabel, 0, 1, 3, 4, 
-						Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
+	elems_table_3->attach (*vbox_examplelabel, 0, 1, 0, 1, 
+						   Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
 
-	elems_table->attach (*altspell_scrollwin, 1, 2, 3, 4, 
-						Gtk::FILL, Gtk::FILL, 0, 0);
+	elems_table_3->attach (*example_scrollwin, 1, 2, 0, 1, 
+						   Gtk::FILL|Gtk::EXPAND, Gtk::FILL|Gtk::EXPAND, 0, 0);
 
 	/*-----------------------------------------------------------*/
-	elems_table->attach (*alignment7, 0, 1, 4, 5, 
-						 Gtk::FILL, Gtk::FILL, 0, 0);
+	elems_table_3->attach (*hbox_asb, 1, 2, 1, 2, 
+						   Gtk::FILL|Gtk::EXPAND, Gtk::SHRINK, 0, 0);
+	/*-----------------------------------------------------------*/
 
-#ifdef IS_HILDON
-    /** Maemo has a narrow screen and thus requires
+	/** Pane 1 holds top elems_table and Pane 2.
+	 *	Pane 2 holds both middle and the bottom elems_table.
+	 */
+	paned_2->pack1 (*elems_table_2, false, false);
+	paned_2->pack2 (*elems_table_3, true,  false);
+
+	paned_1->pack1 (*elems_table_1, true,  false);
+	paned_1->pack2 (*paned_2,       true,  false);
+
+    /** Both PDAs and Maemo have a narrow screen and thus requires
 	 *  smart packing.
 	 */
+#if defined(IS_HILDON) || defined (IS_PDA)
 	Gtk::ScrolledWindow* scrollw = Gtk::manage (new Gtk::ScrolledWindow);
-	scrollw->set_flags (Gtk::CAN_FOCUS);
-	scrollw->set_shadow_type (Gtk::SHADOW_NONE);
-	scrollw->set_policy (Gtk::POLICY_AUTOMATIC, Gtk::POLICY_ALWAYS);
-	scrollw->property_window_placement ().set_value (Gtk::CORNER_TOP_LEFT);
-
-	scrollw->add (*elems_table);
+	SET_CV_SCROLLWIN (scrollw, paned_1);
 	vbox_all->pack_start (*scrollw);
 #else                                      // Regular desktop 
-	vbox_all->pack_start (*elems_table);
+	vbox_all->pack_start (*paned_1);
 #endif
 
-	/**-------------------------**
-	 ** Create and pack buttons **
-	 **-------------------------**
+	/**--------------------------------------------------**
+	 ** Pack [Help] [Cancel][+ Add][Add-n-Close] Buttons **
+	 **--------------------------------------------------**
 	 */
 	Gtk::HBox* hbbox = Gtk::manage(new Gtk::HBox(false, 0));
 	Gtk::HSeparator* hseparator = manage (new Gtk::HSeparator);
+
+#if !defined(IS_PDA)
 	Gtk::HButtonBox*  hbuttonbox = manage (new Gtk::HButtonBox);
-    hbuttonbox->set_homogeneous ();
-    hbuttonbox->set_spacing (8);
+
+    hbuttonbox->set_homogeneous (false);
+    hbuttonbox->set_spacing (0);
 	hbuttonbox->set_layout (Gtk::BUTTONBOX_END);
 	hbuttonbox->set_name ("CardView_CtrlButtonBox");
 
-	hbuttonbox->pack_end   (*m_cancel_button,      false, false, 1);
-	hbuttonbox->pack_end   (*m_add_button,         false, false, 1);
-	hbuttonbox->pack_end   (*m_add_n_close_button, false, false, 1);
+	hbuttonbox->pack_end (*m_count_button,       Gtk::PACK_SHRINK, 0);
+	hbuttonbox->pack_end (*m_cancel_button,      Gtk::PACK_SHRINK, 0);
+	hbuttonbox->pack_end (*m_add_button,         Gtk::PACK_SHRINK, 0);
+	hbuttonbox->pack_end (*m_add_n_close_button, Gtk::PACK_SHRINK, 0);
 
-	hbbox->pack_start (*m_help_button, Gtk::PACK_SHRINK, 5);
+	hbbox->pack_start (*m_help_button, Gtk::PACK_SHRINK, 0);
 	hbbox->pack_start (*hbuttonbox);
+#else
+	hbbox->pack_start (*m_help_button,        Gtk::PACK_SHRINK, 0);
+	hbbox->pack_end   (*m_add_button,         Gtk::PACK_SHRINK, 0);
+	hbbox->pack_end   (*m_add_n_close_button, Gtk::PACK_SHRINK, 0);
+	hbbox->pack_end   (*m_cancel_button,      Gtk::PACK_SHRINK, 0);
+	hbbox->pack_end   (*m_count_button,       Gtk::PACK_SHRINK, 0);
+#endif	
 
-	vbox_all->pack_start (*hseparator, Gtk::PACK_SHRINK, 3);
-	vbox_all->pack_start (*hbbox, Gtk::PACK_SHRINK, 5);
+	vbox_all->pack_start (*hseparator, Gtk::PACK_SHRINK, 0);
+	vbox_all->pack_start (*hbbox,      Gtk::PACK_SHRINK, 0);
 
 	add (*vbox_all);
 
@@ -494,18 +601,20 @@ CardView (VCard* card_)
 		m_front_text   ->get_buffer ()->set_text (m_card->get_question ());
 		m_back_text    ->get_buffer ()->set_text (m_card->get_answer   ());
 		m_example_text ->get_buffer ()->set_text (m_card->get_example  ());
-		m_altspell_text->get_buffer ()->set_text (m_card->get_alt_spelling  ());
+		m_asf_text->get_buffer ()->set_text (m_card->get_alt_spelling (FRONT));
+		m_asb_text->get_buffer ()->set_text (m_card->get_alt_spelling (BACK));
 	}
 
 	/** Set Widget names - very helpful with debugging!
 	 */
-	m_front_text->set_name    ("FrontText");
-	m_back_text ->set_name    ("BackText");
-	m_example_text->set_name  ("ExampleText");
-	m_altspell_text->set_name ("AltSpellText");
-	m_help_button->set_name   ("HelpButton");
-	m_cancel_button->set_name ("CancelButton");
-	m_add_button->set_name    ("AddButton");
+	m_front_text        ->set_name ("FrontText");
+	m_back_text         ->set_name ("BackText");
+	m_example_text      ->set_name ("ExampleText");
+	m_asf_text          ->set_name ("AltSpellFrontText");
+	m_asb_text          ->set_name ("AltSpellBackText");
+	m_help_button       ->set_name ("HelpButton");
+	m_cancel_button     ->set_name ("CancelButton");
+	m_add_button        ->set_name ("AddButton");
 	m_add_n_close_button->set_name ("AddNCloseButton");
 
 	vbox_all->set_name ("CardView_VBoxAll");
@@ -556,6 +665,7 @@ run (bool disable_cancel_)
  *   - DeckView::on_add_clicked ()
  *
  * Possible values returned by run():
+ *
  *    <Cancel> - Gtk::RESPONSE_CANCEL
  *    <Add>    - Gtk::RESPONSE_APPLY
  *    <Ok>     - Gtk::RESPONSE_OK
@@ -568,11 +678,11 @@ run (DeckView& deckview_)
 
 	m_syntax_error = false;
 	take_snapshot ();
+	set_cards_count (deckview_);
 
 #ifdef IS_HILDON
 	GRANULE->register_appview (this);
 	Gtk::Main::run (*HILDONAPPWIN);
-
 #else
 	Gtk::Main::run (*this);
 #endif
@@ -617,6 +727,7 @@ run (DeckView& deckview_)
 			break;
 		}
 		deckview_.add_new_card (dynamic_cast<Card*>(m_card));
+		bump_cards_count ();
 	}
 
   done:
@@ -655,9 +766,9 @@ on_key_press_event (GdkEventKey* event_)
 	DL((GEOM,"key_pressed = %d\n", event_->keyval));
 
 	/** Trap and disable ESC key event.
-		The values for all keysyms are found 
-		in $prefix/gtk-2.0/gdk/gdkkeysyms.h
-	*/
+	 *	The values for all keysyms are found 
+	 *	in $prefix/gtk-2.0/gdk/gdkkeysyms.h
+	 */
 	if (event_->keyval == GDK_Escape) { 
 		return false;
 	}
@@ -677,10 +788,25 @@ CardView::
 on_addnclose_clicked  ()
 {
 	trace_with_mask("CardView::on_addnclose_clicked",GUITRACE);
-	
+
 	m_response = Gtk::RESPONSE_OK;
 
-	if (!Granule::check_markup (m_front_text->get_buffer ()->get_text ())) {
+	Glib::ustring text = m_front_text->get_buffer ()->get_text ();
+
+	if (text.length () == 0) 
+	{
+		Gtk::MessageDialog err0 (
+	_("<b>Front:</b> text field <span foreground=\"red\">IS EMPTY</span>!\n"
+	  "This is an illegal syntax.\n"
+	  "You must fill it with (unique) text.\n"),
+	true, Gtk::MESSAGE_ERROR);
+		err0.run ();
+		m_syntax_error = true;
+		return;
+	}
+
+	if (!Granule::check_markup (text)) 
+	{
 		Gtk::MessageDialog err1 (
 			_("FRONT field contains illegal markup tag(s)."),
 			false, Gtk::MESSAGE_ERROR);
@@ -689,7 +815,8 @@ on_addnclose_clicked  ()
 		return;
 	}
 
-	if (!Granule::check_markup (m_back_text->get_buffer ()->get_text ())) {
+	if (!Granule::check_markup (m_back_text->get_buffer ()->get_text ())) 
+	{
 		Gtk::MessageDialog err2 (
 			_("BACK field contains illegal markup tag(s)."),
 			false, Gtk::MESSAGE_ERROR);
@@ -698,23 +825,26 @@ on_addnclose_clicked  ()
 		return;
 	}
 
-	if (!Granule::check_markup (m_example_text->get_buffer ()->get_text ())) {
-		Gtk::MessageDialog err2 (
+	if (!Granule::check_markup (m_example_text->get_buffer ()->get_text ())) 
+	{
+		Gtk::MessageDialog err3 (
 			_("EXAMPLE field contains illegal markup tag(s)."),
 			false, Gtk::MESSAGE_ERROR);
-		err2.run ();
+		err3.run ();
 		m_syntax_error = true;
 		return;
 	}
 
-	/** Transfer data from the widget to the card
+	/** Transfer data from the widget to the Card object
 	 */
 	if (m_card != NULL && text_modified ()) 
 	{
 		m_card->set_question     (m_front_text   ->get_buffer ()->get_text ());
 		m_card->set_answer       (m_back_text    ->get_buffer ()->get_text ());
 		m_card->set_example      (m_example_text ->get_buffer ()->get_text ());
-		m_card->set_alt_spelling (m_altspell_text->get_buffer ()->get_text ());
+
+		m_card->set_alt_spelling (FRONT, m_asf_text->get_buffer()->get_text ());
+		m_card->set_alt_spelling (BACK,  m_asb_text->get_buffer()->get_text ());
 	}
 	m_keep_open = false;
 	m_front_text->grab_focus ();
@@ -727,6 +857,7 @@ on_addnclose_clicked  ()
 #else	
 	Gtk::Main::quit ();
 #endif
+	/* done */
 }
 
 void
@@ -738,7 +869,8 @@ take_snapshot ()
 	m_snapshot_front    = m_front_text   ->get_buffer ()->get_text ();
 	m_snapshot_back     = m_back_text    ->get_buffer ()->get_text ();
 	m_snapshot_example  = m_example_text ->get_buffer ()->get_text ();
-	m_snapshot_altspell = m_altspell_text->get_buffer ()->get_text ();
+	m_snapshot_asf      = m_asf_text     ->get_buffer ()->get_text ();
+	m_snapshot_asb      = m_asb_text     ->get_buffer ()->get_text ();
 }
 
 bool
@@ -748,7 +880,8 @@ text_modified ()
 	return (m_snapshot_front   != m_front_text   ->get_buffer ()->get_text () ||
 			m_snapshot_back    != m_back_text    ->get_buffer ()->get_text () ||
 			m_snapshot_example != m_example_text ->get_buffer ()->get_text () ||
-			m_snapshot_altspell != m_altspell_text->get_buffer ()->get_text ());
+			m_snapshot_asf     != m_asf_text     ->get_buffer ()->get_text () ||
+			m_snapshot_asb     != m_asb_text     ->get_buffer ()->get_text ());
 }
 
 void
@@ -779,12 +912,14 @@ clear_dialog ()
 	m_front_text  ->get_buffer ()->set_text ("");
 	m_back_text   ->get_buffer ()->set_text ("");
 	m_example_text->get_buffer ()->set_text ("");
-	m_altspell_text->get_buffer ()->set_text ("");
+	m_asf_text    ->get_buffer ()->set_text ("");
+	m_asb_text    ->get_buffer ()->set_text ("");
 
 	m_snapshot_front    = "";
 	m_snapshot_back     = "";
 	m_snapshot_example  = "";
-	m_snapshot_altspell = "";
+	m_snapshot_asf      = "";
+	m_snapshot_asb      = "";
 }
 
 void
@@ -807,3 +942,35 @@ hildon_im_close_cb ()
 	set_size_request (672, 396);
 }
 #endif
+
+void
+CardView::
+set_cards_count (DeckView& deckview_)
+{
+	static char buf [32];
+	m_cards_count = deckview_.get_deck_size ();
+
+#ifdef IS_PDA
+	sprintf (buf, "%d", m_cards_count);
+#else
+	sprintf (buf, "%d cards", m_cards_count);
+#endif
+	m_count_button->set_label (buf);
+}
+
+void
+CardView::
+bump_cards_count ()
+{
+	static char buf [32];
+
+	++m_cards_count;
+
+#ifdef IS_PDA
+	sprintf (buf, "%d", m_cards_count);
+#else
+	sprintf (buf, "%d cards", m_cards_count);
+#endif
+
+	m_count_button->set_label (buf);
+}
