@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              CardView.h
 //------------------------------------------------------------------------------
-// $Id: CardView.h,v 1.20 2006/09/06 02:07:59 vlg Exp $
+// $Id: CardView.h,v 1.22 2006/12/18 03:33:14 vlg Exp $
 //
 //  Copyright (c) 2004-2005 by Vladislav Grinchenko
 //
@@ -79,6 +79,8 @@ private:
 	void clear_dialog ();
 
 	void take_snapshot ();
+	void set_cards_count (DeckView& deckview_);
+	void bump_cards_count ();
 	bool text_modified ();
 
 private:
@@ -90,6 +92,7 @@ private:
 	// -*- GUI components -*-
 	//
 	Gtk::Button*    m_cancel_button;
+	Gtk::Button*    m_count_button;	          // functions as a label
 	Gtk::Button*    m_add_n_close_button;
 	Gtk::Button*    m_add_button;
 	Gtk::Button*    m_help_button;
@@ -97,18 +100,22 @@ private:
 	Gtk::TextView*  m_front_text;
 	Gtk::TextView*  m_back_text;
 	Gtk::TextView*  m_example_text;
-	Gtk::TextView*  m_altspell_text;
+	Gtk::TextView*  m_asf_text;	    // Alt.Spell Front
+	Gtk::TextView*  m_asb_text;	    // Alt.Spell Back
 
 	// -*- Snapshot to detect true changes -*-
 	//
 	Glib::ustring   m_snapshot_front;
 	Glib::ustring   m_snapshot_back;
 	Glib::ustring   m_snapshot_example;
-	Glib::ustring   m_snapshot_altspell;
+	Glib::ustring   m_snapshot_asf;
+	Glib::ustring   m_snapshot_asb;
 
 	EditControls*   m_edit_controls;
 
 	CardViewHelpDialog m_help_dialog;
+
+	int             m_cards_count; // tied up to m_count_button
 };
 
 
