@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: DeckView.cpp,v 1.48 2006/09/30 20:42:27 vlg Exp $
+// $Id: DeckView.cpp,v 1.54 2007/01/15 04:09:08 vlg Exp $
 //------------------------------------------------------------------------------
 //                            DeckView.cpp
 //------------------------------------------------------------------------------
@@ -38,24 +38,23 @@
 using sigc::bind;
 using sigc::mem_fun;
 
+#define DV_SETUP_LABEL(l) \
+	(l)->set_alignment  (0.5,0.5); \
+	(l)->set_padding    (0,0); \
+	(l)->set_justify    (Gtk::JUSTIFY_LEFT); \
+	(l)->set_line_wrap  (false); \
+	(l)->set_use_markup (false); \
+	(l)->set_selectable (false); 
+
+//------------------------------------------------------------------------------
+// DeckView implementation
+//------------------------------------------------------------------------------
 DeckView::
 DeckView (VDeck& deck_, int selected_row_) 
 		:
 #ifdef IS_HILDON
 	Hildon::AppView    ("Deck View"),
 #endif
-	m_add_button       (manage (new Gtk::Button (Gtk::StockID ("gtk-add")))),
-	m_edit_button      (manage (new ButtonWithImageLabel (
-									Gtk::Stock::DND, "Edit"))),
-	m_delete_button    (manage (new Gtk::Button (Gtk::StockID("gtk-delete")))),
-	m_save_deck_button (manage (new ButtonWithImageLabel(
-									Gtk::Stock::SAVE,"Save Deck"))),
-	m_more_button      (manage (new ButtonWithImageLabel (
-									Gtk::Stock::DIALOG_INFO, "Deck Info"))),
-	m_card_up_button   (manage (new Gtk::Button (Gtk::Stock::GO_UP))),
-	m_card_down_button (manage (new Gtk::Button (Gtk::Stock::GO_DOWN))),
-	m_add_card_to_cardbox (manage (new ButtonWithImageLabel (
-									   Gtk::Stock::UNDO, "Add to Box 1 "))),
 	m_label_count      (manage (new Gtk::Label ("0 cards"))),
 	m_deck             (deck_),
 	m_row_idx          (selected_row_),
@@ -63,8 +62,72 @@ DeckView (VDeck& deck_, int selected_row_)
 {
 	trace_with_mask("DeckView::DeckView", GUITRACE);
 
+	m_actgroup = Gtk::ActionGroup::create ();
+
+	m_actgroup->add (Gtk::Action::create ("Add", 
+										  Gtk::Stock::ADD,
+										  "Add",
+										  "Add new card"),
+					 mem_fun (*this, &DeckView::on_add_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("Edit", 
+										  Gtk::Stock::DND,
+										  "Edit",
+										  "Edit selected card"),
+					 mem_fun (*this, &DeckView::on_edit_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("Delete", 
+#ifdef WIN32
+										  Gtk::Stock::CUT,
+#else
+										  Gtk::Stock::DELETE,
+#endif
+										  "Delete",
+										  "Delete selected card"),
+					 mem_fun (*this, &DeckView::on_delete_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("Go_Up", 
+										  Gtk::Stock::GO_UP,
+										  "Move Up",
+										  "Move selecte card up"),
+					 mem_fun (*this, &DeckView::on_card_up_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("Go_Down", 
+										  Gtk::Stock::GO_DOWN,
+										  "Move Down",
+										  "Move selected card down"),
+					 mem_fun (*this, &DeckView::on_card_down_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("Add_Box1", 
+										  Gtk::Stock::UNDO,
+										  "Add To Box1",
+										  "Add selected card to Box1"),
+					 mem_fun (*this, &DeckView::on_add_card_to_cardbox));
+
+	m_actgroup->add (Gtk::Action::create ("Deck_Info", 
+										  Gtk::Stock::DIALOG_INFO,
+										  "Deck Info",
+										  "Deck Information"),
+					 mem_fun (*this, &DeckView::on_more_clicked));
+
+	m_actgroup->add (Gtk::Action::create ("Save_Deck", 
+										  Gtk::Stock::SAVE,
+										  "Save",
+										  "Save deck"),
+					 mem_fun (*this, &DeckView::on_save_deck_clicked));
+
+    m_uimanager = Gtk::UIManager::create ();
+    m_uimanager->insert_action_group (m_actgroup);
+
+#ifndef IS_HILDON
+    add_accel_group (m_uimanager->get_accel_group ());
+#endif
+
+
+	/********************************
+	 * Setup dialog attributes
+	 ********************************/
 	set_title("Deck View");
-	set_border_width (2);
 
 #ifdef IS_HILDON
 	/**
@@ -76,81 +139,66 @@ DeckView (VDeck& deck_, int selected_row_)
 	add (*m_vbox);
 	set_size_request (672, 396);
 #else
-	set_resizable(true);
-	set_position (Gtk::WIN_POS_CENTER_ALWAYS);
-	set_size_request (633, 410);	// width; height 
 	set_modal (true);
-	set_transient_for (*MAINWIN);
-#endif
-
-	Gtk::Label* label_total = manage (new Gtk::Label ("Total: "));
-	Gtk::VBox* vbox         = manage (new Gtk::VBox(false, 0));
-	Gtk::HBox* h_infobox    = manage (new Gtk::HBox(false, 0));
-	Gtk::HBox* h_viewbox    = manage (new Gtk::HBox(false, 0));
-
-	m_add_button         ->set_flags  (Gtk::CAN_FOCUS);
-	m_add_button         ->set_relief (Gtk::RELIEF_NORMAL);
-	m_edit_button        ->set_flags  (Gtk::CAN_FOCUS);
-	m_edit_button        ->set_relief (Gtk::RELIEF_NORMAL);
-    m_save_deck_button   ->set_flags  (Gtk::CAN_FOCUS);
-	m_save_deck_button   ->set_relief (Gtk::RELIEF_NORMAL);
-	m_delete_button      ->set_flags  (Gtk::CAN_FOCUS);
-	m_delete_button      ->set_relief (Gtk::RELIEF_NORMAL);
-	m_more_button        ->set_flags  (Gtk::CAN_FOCUS);
-	m_more_button        ->set_relief (Gtk::RELIEF_NORMAL);
-	m_card_down_button   ->set_flags  (Gtk::CAN_FOCUS);
-	m_card_down_button   ->set_relief (Gtk::RELIEF_NORMAL);
-	m_card_up_button     ->set_flags  (Gtk::CAN_FOCUS);
-	m_card_up_button     ->set_relief (Gtk::RELIEF_NORMAL);
-	m_add_card_to_cardbox->set_flags  (Gtk::CAN_FOCUS);
-	m_add_card_to_cardbox->set_relief (Gtk::RELIEF_NORMAL);
-
-	vbox->set_size_request (122,-1);
-	vbox->pack_start (*m_add_button,          Gtk::PACK_SHRINK, 4);
-	vbox->pack_start (*m_edit_button,         Gtk::PACK_SHRINK, 4);
-	vbox->pack_start (*m_delete_button,       Gtk::PACK_SHRINK, 4);
-
-	vbox->pack_start (*m_card_up_button,      Gtk::PACK_SHRINK, 4);
-	vbox->pack_start (*m_card_down_button,    Gtk::PACK_SHRINK, 4);
-	vbox->pack_start (*m_add_card_to_cardbox, Gtk::PACK_SHRINK, 4);
-
-	vbox->pack_start (*m_more_button,         Gtk::PACK_SHRINK, 4);
-	vbox->pack_start (*m_save_deck_button,    Gtk::PACK_SHRINK, 4);
-
-	label_total->set_alignment  (0.5,0.5);
-	label_total->set_padding    (0,0);
-	label_total->set_justify    (Gtk::JUSTIFY_LEFT);
-	label_total->set_line_wrap  (false);
-	label_total->set_use_markup (false);
-	label_total->set_selectable (false);
-
-	m_label_count->set_alignment  (0.5,0.5);
-	m_label_count->set_padding    (0,0);
-	m_label_count->set_justify    (Gtk::JUSTIFY_LEFT);
-	m_label_count->set_line_wrap  (false);
-	m_label_count->set_use_markup (false);
-	m_label_count->set_selectable (false);
-
-	h_infobox->pack_start (*label_total,   Gtk::PACK_SHRINK, 9);
-	h_infobox->pack_start (*m_label_count, Gtk::PACK_SHRINK, 0);
-
-	/** <Close> button
-	 */
-#ifdef IS_HILDON
-	m_close_button = manage (new ButtonWithImageLabel (Gtk::Stock::CLOSE, 
-													   "Close"));
+//	set_position (Gtk::WIN_POS_CENTER_ALWAYS);
+#ifdef IS_PDA
+	set_resizable(false);
+	Gdk::Geometry dp_geometry =	{ 240, 320,	240, 320, 240, 320,	-1, -1,	-1, -1};
+	set_geometry_hints (*this, dp_geometry, 
+				Gdk::HINT_MIN_SIZE | Gdk::HINT_MAX_SIZE | Gdk::HINT_BASE_SIZE);
 #else
-	m_close_button = add_button (Gtk::StockID ("gtk-close"), 
-								 Gtk::RESPONSE_CLOSE );
-#endif
-	m_close_button->set_flags  (Gtk::CAN_FOCUS);
-	m_close_button->set_relief (Gtk::RELIEF_NORMAL);
+	set_resizable(true);
+	set_size_request (400, 370);	// width; height 
+	set_border_width (2);
+	set_transient_for (*MAINWIN);
+#endif // (!IS_PDA)
+#endif // (!IS_HILDON)
 
-	m_close_button->signal_clicked ().connect (
-		mem_fun (*this, &DeckView::on_close_clicked));
+	/********************************
+	 * Setup toolbar                
+	 ********************************/
+	try {
+		Glib::ustring ui_info =
+			"<ui>"
+			"  <toolbar name='DV_Controls'>"
+			"     <toolitem action='Add'/>"
+			"     <toolitem action='Edit'/>"
+			"     <toolitem action='Delete'/>"
+			"     <toolitem action='Go_Up'/>"
+			"     <toolitem action='Go_Down'/>"
+			"     <toolitem action='Add_Box1'/>"
+			"     <toolitem action='Deck_Info'/>"
+			"     <toolitem action='Save_Deck'/>"
+			"  </toolbar>"
+			"</ui>";
+		m_uimanager->add_ui_from_string (ui_info);
+    }
+    catch (const Glib::Error& ex_) {
+		DL((GRAPP,"Building menues failed (%s)\n", ex_.what ().c_str ()));
+    }
+		
+	m_add_button          = m_uimanager->get_widget ("/DV_Controls/Add");
+	m_edit_button         = m_uimanager->get_widget ("/DV_Controls/Edit");
+	m_delete_button       = m_uimanager->get_widget ("/DV_Controls/Delete");
+	m_card_up_button      = m_uimanager->get_widget ("/DV_Controls/Go_Up");
+	m_card_down_button    = m_uimanager->get_widget ("/DV_Controls/Go_Down");
+	m_add_card_to_cardbox = m_uimanager->get_widget ("/DV_Controls/Add_Box1");
+	m_more_button         = m_uimanager->get_widget ("/DV_Controls/Deck_Info");
+	m_save_deck_button    = m_uimanager->get_widget ("/DV_Controls/Save_Deck");
 
-	/** List View 
-	 */
+    Gtk::Widget* controls = m_uimanager->get_widget ("/DV_Controls");
+    Gtk::Toolbar* ctrl_toolbar = dynamic_cast<Gtk::Toolbar*> (controls);
+
+    ctrl_toolbar->set_tooltips (true);
+	ctrl_toolbar->set_show_arrow ();
+    ctrl_toolbar->set_orientation (Gtk::ORIENTATION_HORIZONTAL);
+    ctrl_toolbar->set_toolbar_style (Gtk::TOOLBAR_ICONS);
+
+	get_vbox ()->pack_start (*ctrl_toolbar, Gtk::PACK_SHRINK, 0);
+
+	/**************************
+	 * List View 
+	 **************************/
 	m_scrollwin = manage (new Gtk::ScrolledWindow);
 	m_scrollwin->set_shadow_type (Gtk::SHADOW_ETCHED_OUT);
 	m_scrollwin->set_policy (Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
@@ -187,8 +235,9 @@ DeckView (VDeck& deck_, int selected_row_)
 	m_list_store_ref->set_sort_func (
 		1, (mem_fun (*this, &DeckView::on_sort_compare_back)));
 
-	/** Create TreeView
-	 */
+	/******************************
+	 * Create TreeView
+	 ******************************/
 	DL((GRAPP,"Create TreeView\n"));
 	m_tree_view.set_model (m_list_store_ref);
 	m_tree_view.set_rules_hint ();
@@ -196,8 +245,9 @@ DeckView (VDeck& deck_, int selected_row_)
 	Gtk::TreeView::Column* column;
 	Gtk::CellRendererText* renderer;
 
-	/** Front: Column
-	 */
+	/******************************
+	 * Front: Column
+	 ******************************/
 	column = Gtk::manage (new Gtk::TreeView::Column ("Front"));
 	m_tree_view.append_column (*column);
 	renderer = Gtk::manage (new Gtk::CellRendererText);
@@ -207,18 +257,24 @@ DeckView (VDeck& deck_, int selected_row_)
 						   m_columns.m_front_row);
 						  
 	column->set_sizing (Gtk::TREE_VIEW_COLUMN_FIXED);
-	column->set_fixed_width (140);
 	column->set_resizable ();
+#ifdef IS_PDA
+	column->set_fixed_width (125);
+#else
+	column->set_fixed_width (250);
+#endif
 
-	/** Enable column sorting
-	 */
+	/******************************
+	 * Enable column sorting
+	 ******************************/
 	column->set_sort_column_id (m_columns.m_front_row);
 	column->set_sort_indicator (true);
 	column->signal_clicked ().connect (
 		mem_fun (*this, &DeckView::on_column_sorted));
 
-	/** Back: Column
-	 */
+	/******************************
+	 * Back: Column
+	 ******************************/
 	column = Gtk::manage (new Gtk::TreeView::Column ("Back"));
 	m_tree_view.append_column (*column);
 	renderer = Gtk::manage (new Gtk::CellRendererText);
@@ -230,8 +286,9 @@ DeckView (VDeck& deck_, int selected_row_)
 	column->set_sizing (Gtk::TREE_VIEW_COLUMN_FIXED);
 	column->set_fixed_width (140);
 
-	/** Enable column sorting
-	 */
+	/****************************
+	 * Enable column sorting
+	 ****************************/
 	column->set_sort_column_id (m_columns.m_back_row);
 	column->set_sort_indicator (true);
 	column->signal_clicked ().connect (
@@ -242,35 +299,50 @@ DeckView (VDeck& deck_, int selected_row_)
 	m_tree_sel_ref->signal_changed ().connect (
 		mem_fun (*this, &DeckView::on_card_selected));
 
+	m_tree_view.add_events (Gdk::BUTTON_PRESS_MASK);
+	m_tree_view.signal_button_press_event ().connect (
+		sigc::mem_fun (*this, &DeckView::on_mouse_click), false);
+
 	m_scrollwin->add (m_tree_view);
 
-	/** Initialize callbacks 
-	 */
-	m_add_button->signal_clicked ().connect (
-		mem_fun (*this, &DeckView::on_add_clicked));
-	m_edit_button->signal_clicked ().connect (
-		mem_fun (*this, &DeckView::on_edit_clicked));
-	m_save_deck_button->signal_clicked ().connect (
-		mem_fun (*this, &DeckView::on_save_deck_clicked));
-	m_delete_button->signal_clicked ().connect (
-		mem_fun (*this, &DeckView::on_delete_clicked));
-	m_more_button->signal_clicked ().connect (
-		mem_fun (*this, &DeckView::on_more_clicked));
-	m_card_up_button->signal_clicked ().connect (
-		mem_fun (*this, &DeckView::on_card_up_clicked));
-	m_card_down_button->signal_clicked ().connect (
-		mem_fun (*this, &DeckView::on_card_down_clicked));
-	m_add_card_to_cardbox->signal_clicked ().connect (
-		mem_fun (*this, &DeckView::on_add_card_to_cardbox));
-
-	/** Pack 'em all
-	 */
-	h_viewbox->pack_start (*m_scrollwin);
-	h_viewbox->pack_start (*vbox, Gtk::PACK_SHRINK, 8);
-
+	/*****************************
+	 * Pack the cards list
+	 *****************************/
+	Gtk::HBox* h_viewbox = manage (new Gtk::HBox(false, 0));
+	h_viewbox->pack_start (*m_scrollwin, Gtk::PACK_EXPAND_WIDGET, 0);
 	get_vbox ()->pack_start (*h_viewbox);
+
+	/***********************************
+	 *  h_infobox holds:
+	 *  -------------------------
+	 *      [Total:] [XXX cards] 
+	 ***********************************/
+	Gtk::Label* label_total = manage (new Gtk::Label ("Total: "));
+	Gtk::HBox* h_infobox    = manage (new Gtk::HBox(false, 0));
+
+	DV_SETUP_LABEL(label_total);
+	DV_SETUP_LABEL(m_label_count);
+
+	h_infobox->pack_start (*label_total,   Gtk::PACK_SHRINK, 9);
+	h_infobox->pack_start (*m_label_count, Gtk::PACK_SHRINK, 0);
+
 	get_vbox ()->pack_start (*h_infobox, Gtk::PACK_SHRINK, 4);
 
+	/** <Close> button
+	 */
+#ifdef IS_HILDON  // Handle the button ourselves
+	m_close_button = manage (new ButtonWithImageLabel (Gtk::Stock::CLOSE, 
+													   "Close"));
+#else  // Add button to the Gtk::Dialog
+	m_close_button = add_button (Gtk::StockID ("gtk-close"), 
+								 Gtk::RESPONSE_CLOSE );
+#endif
+	m_close_button->set_flags  (Gtk::CAN_FOCUS);
+	m_close_button->set_relief (Gtk::RELIEF_NORMAL);
+
+	m_close_button->signal_clicked ().connect (
+		mem_fun (*this, &DeckView::on_close_clicked));
+
 #ifdef IS_HILDON
 	/**
 	 * Add separator and [Close] button in the ButtonBox
@@ -282,12 +354,15 @@ DeckView (VDeck& deck_, int selected_row_)
 	hbuttonbox->set_spacing (6);
 	hbuttonbox->set_border_width (6);
 
-	get_vbox ()->pack_start (*hbuttonbox, false, true, 0);
-	hbuttonbox->pack_end (*m_close_button, false, true, 0);
+	Gtk::Label* expander = manage (new Gtk::Label (""));
+	hbuttonbox->pack_start (*expander, Gtk::PACK_EXPAND_WIDGET);
+	hbuttonbox->pack_end (*m_close_button, Gtk::PACK_SHRINK);
+
+	get_vbox ()->pack_start (*hbuttonbox, Gtk::PACK_SHRINK);
+
 #endif
 
 	set_cards_count ();
-
 	show_all ();
 
 	/** Select the same row (card) that the DeckPlayer has selected,
@@ -311,7 +386,7 @@ DeckView (VDeck& deck_, int selected_row_)
 	set_sensitivity ();
 	set_win_name ();
 
-#ifndef IS_HILDON
+#if !defined(IS_HILDON) && !defined (IS_PDA)
 	set_icon_from_file (GRAPPDATDIR "/pixmaps/deckview_32x32.png");
 #endif
 
@@ -328,6 +403,7 @@ DeckView::
 set_cards_count ()
 {
 	std::ostringstream os;
+
 	os << m_deck.size () << " cards.";
 	m_label_count->set_text (os.str ());
 }
@@ -336,18 +412,20 @@ set_cards_count ()
 //                              Callbacks
 //------------------------------------------------------------------------------
 
-#ifdef IS_HILDON
+//#ifdef IS_HILDON
 gint
 DeckView::
 run ()
 {
     trace_with_mask("DeckView::run",GUITRACE);
 
+#ifdef IS_HILDON
 	GRANULE->register_appview (this);
+#endif
 	Gtk::Main::run ();
 	return (Gtk::RESPONSE_NONE);
 }
-#endif
+//#endif
 
 void
 DeckView::
@@ -358,9 +436,9 @@ on_close_clicked ()
 #ifdef IS_HILDON
 	hide ();
 	GRANULE->unregister_appview (this);
+#endif
 	DL ((GRAPP,"Call Main::quit()\n"));
 	Gtk::Main::quit ();
-#endif
 }
 
 void
@@ -422,6 +500,8 @@ on_add_clicked ()
 {
 	trace_with_mask("DeckView::on_add_clicked", GUITRACE);
 
+	hide ();
+
 	CardView cview (new Card);
 
 	int ret = cview.run (*this);
@@ -440,6 +520,7 @@ on_add_clicked ()
 		DL((GRAPP|ASSA::ASSAERR, "Unexpected response = %d, "
 			"see /path/to/gtkmm/dialog.h for details.\n", ret));
 	}
+	show ();
 }
 
 void 
@@ -456,9 +537,9 @@ on_edit_clicked ()
 	Gtk::TreeModel::Row row = *m_list_store_ref->children ()[m_row_idx];
 	VCard* card_iter = row [m_columns.m_card];
 
-#ifdef IS_HILDON
+//#ifdef IS_HILDON
 	hide ();
-#endif
+//#endif
 
 	CardView cview (card_iter);
 	int ret = cview.run ();
@@ -470,9 +551,9 @@ on_edit_clicked ()
 			Granule::trim_multiline(card_iter->get_answer  ());
 	}
 
-#ifdef IS_HILDON
+//#ifdef IS_HILDON
 	show ();
-#endif
+//#endif
 }
 
 void 
@@ -594,9 +675,11 @@ on_more_clicked ()
 #ifdef IS_HILDON
 	DeckInfo deck_info (*HILDONAPPWIN, dynamic_cast<Deck&>(m_deck));
 	deck_info.run ();
+	m_repaint_player = deck_info.appearance_changed ();
 #else
 	DeckInfo deck_info (*this, dynamic_cast<Deck&>(m_deck));
 	deck_info.run ();
+	m_repaint_player = deck_info.appearance_changed ();
 #endif
 
 }
@@ -853,3 +936,27 @@ on_save_deck_clicked ()
 
 	DECKMGR->save_deck_cb ();
 }
+
+/** Called when button is pressed on any of the rows.
+	We filter out everything except for the double-click
+	which brings the CardView dialog to edit selected entry.
+
+	The rest of the events are handled by the default event
+	handler of TreeView afterwards.
+*/
+bool
+DeckView::
+on_mouse_click (GdkEventButton* event_)
+{
+	trace_with_mask("DeckView::on_mouse_click", GUITRACE);
+
+	if (event_->type == GDK_2BUTTON_PRESS
+#ifdef IS_HILDON
+		|| event_->type == GDK_Escape // Hildon round button (Cancel,Close)
+#endif
+		) 
+	{
+		on_edit_clicked ();
+	}
+	return false;
+}
