@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              GrappConf.cpp
 //------------------------------------------------------------------------------
-// $Id: GrappConf.cpp,v 1.4 2006/09/30 20:42:27 vlg Exp $
+// $Id: GrappConf.cpp,v 1.11 2007/01/15 04:09:08 vlg Exp $
 //------------------------------------------------------------------------------
 //  Copyright (c) 2004-2006 by Vladislav Grinchenko
 //
@@ -27,7 +27,7 @@
 
 
 #if !defined(WIN32)
-#  include <libgen.h>				// basename(3), dirname(3)
+#  include <libgen.h>			// basename(3), dirname(3)
 #  include <pwd.h>				// getpwnam(2), getlogin(2)
 #endif
 
@@ -54,7 +54,7 @@ ASSA_DECL_SINGLETON(GrappConf);
  */
 #if defined(WIN32)
 
-char *dirname (char *path_)
+extern char *dirname (const char *path_)
 {
 	static char path [MAX_PATH];
 
@@ -65,7 +65,7 @@ char *dirname (char *path_)
 	return path;
 }
 
-char *basename (char* path_)
+extern char *basename (const char* path_)
 {
 	static char path [MAX_PATH];
 
@@ -118,23 +118,41 @@ dump (int idx_)
 /******************************************************************************/
 GrappConf::
 GrappConf () :
-    m_proj_name         ("untitled.cdf"),
-    m_dont_save_project (false),
-	m_mwin_geometry     (0, 0, 430, 350),
-	m_dplyr_geometry    (0, 0, 583, 350),
-	m_crdview_geometry  (0, 0, 643, 386),
-	m_dplyr_keep_aspect (true),
-	m_dplyr_aspect      (0, 0, 5, 3),
-	m_snd_player_cmd    ("sox"),
-	m_snd_player_args   ("%s -q -t ossdsp -v 1.7  -r 48000 /dev/dsp"),
-	m_config            (NULL),
-	m_remove_dups       (true),
+    m_proj_name           ("untitled.cdf"),
+    m_dont_save_project   (false),
+	m_text_fgcolor        ("black"),
+	m_text_bgcolor        ("white"),
+	m_sep_color           ("pink"),
+	m_front_x_alignment   ("center"),
+	m_front_y_alignment   ("center"),
+	m_front_x_padding     ("4"),
+	m_front_y_padding     ("0"),
+	m_front_justification ("center"),
+	m_back_x_alignment    ("center"),
+	m_back_y_alignment    ("center"),
+	m_back_x_padding      ("1"),
+	m_back_y_padding      ("1"),
+	m_back_justification  ("center"),
+	m_example_x_alignment ("left"),
+	m_example_y_alignment ("top"),
+	m_example_x_padding   ("2"),
+	m_example_y_padding   ("0"),
+	m_example_justification ("left"),
+	m_mwin_geometry       (0, 0, 430, 350),
+	m_dplyr_geometry      (0, 0, 583, 350),
+	m_crdview_geometry    (0, 0, 643, 386),
+	m_dplyr_keep_aspect   (true),
+	m_dplyr_aspect        (0, 0, 5, 3),
+	m_snd_player_cmd      ("sox"),
+	m_snd_player_args     ("%s -q -t ossdsp -v 1.7  -r 48000 /dev/dsp"),
+	m_config              (NULL),
+	m_remove_dups         (true),
 	m_with_relative_paths (false),
-	m_load_time         (ASSA::TimeVal::gettimeofday ()),
-	m_root_x            (0),
-    m_root_y            (0),
+	m_load_time           (ASSA::TimeVal::gettimeofday ()),
+	m_root_x              (0),
+    m_root_y              (0),
 	m_disable_key_shortcuts (false),
-	m_auto_pronounce    (false)
+	m_auto_pronounce      (false)
 {
     trace_with_mask("GrappConf::GrappConf",GUITRACE);
 
@@ -328,11 +346,29 @@ remove_document_history (const string& fullpath_)
  *
  * [Default]
  *    filesel_path=/path/to/
+ *    text_bgcolor="white"
+ *    text_fgcolor="black"
+ *    sep_color="pink"
  *    question_font_desc="Sans 14"
  *    answer_font_desc="Sans 14"
  *    example_font_desc="Sans 14"
  *    input_font_desc="Sans 14"
  *    app_font_desc="Bitstream Vera Sans 12"
+ *    front_x_alignment="center"
+ *    front_y_alignment="center"
+ *    front_x_padding="4"
+ *    front_y_padding="0"
+ *    front_justification="center"
+ *    back_x_alignment="center"
+ *    back_y_alignment="center"
+ *    back_x_padding="0"
+ *    back_y_padding="0"
+ *    back_justification="center"
+ *    example_x_alignment="center"
+ *    example_y_alignment="center"
+ *    example_x_padding="2"
+ *    example_y_padding="0"
+ *    example_justification="left"
  *    mainwin_geometry=430x350
  *    cardview_geometry_height=643x386
  *    deckplayer_geometry=583x350
@@ -354,21 +390,22 @@ load_config (int secs_in_day_, int secs_in_week_)
 {
     trace_with_mask("GrappConf::load_config",GUITRACE);
 
-    Glib::ustring usfont;
-    std::string s;
+    Glib::ustring ustr;
+    string s;
     int idx;
     IniFile::const_config_iterator section;
 
     if (m_config == NULL) {
 		s = GRANULE->get_config_file ();
-		if (s.size () == 0) {
+		if (s.size () == 0 || s == "none") {
 			s = GRANULE->get_default_config_file ();
 		}
 		m_config = new IniFile (s);
 		if (m_config->load () < 0) {
-			DL((APP, "Failed to load configuration file!\n"));
+			DL((GRAPP, "Failed to load configuration file!\n"));
 			return;
 		}
+		DL ((GRAPP,"Loaded configuration from \"%s\"\n", s.c_str ()));
 		s = "";
     }
     /** Load [History] section
@@ -424,8 +461,8 @@ load_config (int secs_in_day_, int secs_in_week_)
     if (section != m_config->sect_end ()) {
 		DL((APP,"Scanning [History] section ...\n"));
 		IniFile::const_tuple_iterator k = (*section).second.begin ();
-		std::string lhs;
-		std::string rhs;
+		string lhs;
+		string rhs;
 
 		while (k != (*section).second.end ()) { 
 			idx = ::atoi ((*k).first.c_str ()) - 1;
@@ -484,34 +521,125 @@ load_config (int secs_in_day_, int secs_in_week_)
 	}
 
     DL((APP,"m_filesel_path     = \"%s\"\n", m_filesel_path.c_str ()));
-    DL((APP,"m_font_desc        = \"%s\"\n", usfont.c_str ()));
+    DL((APP,"m_font_desc        = \"%s\"\n", ustr.c_str ()));
     DL((APP,"m_snd_player_cmd   = \"%s\"\n", m_snd_player_cmd.c_str ()));
     DL((APP,"m_snd_player_args  = \"%s\"\n", m_snd_player_args.c_str ()));
     DL((APP,"m_snd_archive_path = \"%s\"\n", m_snd_archive_path.c_str ()));
 
-    usfont = m_config->get_value ("Default", "question_font_desc");
-    if (! usfont.empty ()) {
-		set_question_font (usfont);
+	ustr = m_config->get_value ("Default", "text_bgcolor");
+	if (! ustr.empty ()) {
+		text_bgcolor (ustr);
+		DL((APP,"m_text_bgcolor     = \"%s\"\n", m_text_bgcolor.c_str ()));
+	}
+
+	ustr = m_config->get_value ("Default", "text_fgcolor");
+	if (! ustr.empty ()) {
+		text_fgcolor (ustr);
+		DL((APP,"m_text_fgcolor     = \"%s\"\n", m_text_fgcolor.c_str ()));
+	}
+	ustr = m_config->get_value ("Default", "sep_color");
+	if (! ustr.empty ()) {
+		sep_color (ustr);
+	}
+
+    ustr = m_config->get_value ("Default", "question_font_desc");
+    if (! ustr.empty ()) {
+		set_question_font (ustr);
     }
 
-    usfont = m_config->get_value ("Default", "answer_font_desc");
-    if (! usfont.empty ()) {
-		set_answer_font (usfont);
+    ustr = m_config->get_value ("Default", "answer_font_desc");
+    if (! ustr.empty ()) {
+		set_answer_font (ustr);
     }
 
-    usfont = m_config->get_value ("Default", "example_font_desc");
-    if (! usfont.empty ()) {
-		set_example_font (usfont);
+    ustr = m_config->get_value ("Default", "example_font_desc");
+    if (! ustr.empty ()) {
+		set_example_font (ustr);
     }
 
-    usfont = m_config->get_value ("Default", "input_font_desc");
-    if (! usfont.empty ()) {
-		set_input_font (usfont);
+    ustr = m_config->get_value ("Default", "input_font_desc");
+    if (! ustr.empty ()) {
+		set_input_font (ustr);
     }
 
-    usfont = m_config->get_value ("Default", "app_font_desc");
-    if (! usfont.empty ()) {
-		set_app_font (usfont);
+    ustr = m_config->get_value ("Default", "app_font_desc");
+    if (! ustr.empty ()) {
+		set_app_font (ustr);
+    }
+
+    s = m_config->get_value ("Default", "front_x_alignment");
+    if (! s.empty ()) {
+		m_front_x_alignment = s;
+    }
+
+    s = m_config->get_value ("Default", "front_y_alignment");
+    if (! s.empty ()) {
+		m_front_y_alignment = s;
+    }
+
+    s = m_config->get_value ("Default", "front_x_padding");
+    if (! s.empty ()) {
+		m_front_x_padding = s;
+    }
+
+    s = m_config->get_value ("Default", "front_y_padding");
+    if (! s.empty ()) {
+		m_front_y_padding = s;
+    }
+
+    s = m_config->get_value ("Default", "front_justification");
+    if (! s.empty ()) {
+		m_front_justification = s;
+    }
+
+    s = m_config->get_value ("Default", "back_x_alignment");
+    if (! s.empty ()) {
+		m_back_x_alignment = s;
+    }
+
+    s = m_config->get_value ("Default", "back_y_alignment");
+    if (! s.empty ()) {
+		m_back_y_alignment = s;
+    }
+
+    s = m_config->get_value ("Default", "back_x_padding");
+    if (! s.empty ()) {
+		m_back_x_padding = s;
+    }
+
+    s = m_config->get_value ("Default", "back_y_padding");
+    if (! s.empty ()) {
+		m_back_y_padding = s;
+    }
+
+    s = m_config->get_value ("Default", "back_justification");
+    if (! s.empty ()) {
+		m_back_justification = s;
+    }
+
+    s = m_config->get_value ("Default", "example_x_alignment");
+    if (! s.empty ()) {
+		m_example_x_alignment = s;
+    }
+
+    s = m_config->get_value ("Default", "example_y_alignment");
+    if (! s.empty ()) {
+		m_example_y_alignment = s;
+    }
+
+    s = m_config->get_value ("Default", "example_x_padding");
+    if (! s.empty ()) {
+		m_example_x_padding = s;
+    }
+
+    s = m_config->get_value ("Default", "example_y_padding");
+    if (! s.empty ()) {
+		m_example_y_padding = s;
+    }
+
+    s = m_config->get_value ("Default", "example_justification");
+    if (! s.empty ()) {
+		m_example_justification = s;
     }
 
     s = m_config->get_value ("Default", "mainwin_geometry");
@@ -568,7 +696,7 @@ save_config ()
 {
     trace_with_mask("GrappConf::save_config",GUITRACE);
 
-	std::string s;
+	string s;
 	const char* fname;
 
     if (m_config == NULL) {
@@ -652,28 +780,81 @@ save_config ()
 		iter++, position++;
     }
 
-    Glib::ustring usfont;
+    Glib::ustring ustr;
     m_config->set_pair ("Default", 
 						IniFile::tuple_type("filesel_path", m_filesel_path));
-
+	/** Text colors
+	 */
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("text_fgcolor",	m_text_fgcolor));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("text_bgcolor",	m_text_bgcolor));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("sep_color",	m_sep_color));
     /** Fonts
      */
-    usfont = m_question_font_desc.to_string ();
+    ustr = m_question_font_desc.to_string ();
     m_config->set_pair ("Default", 
-						IniFile::tuple_type("question_font_desc",usfont));
-    usfont = m_answer_font_desc.to_string ();
+						IniFile::tuple_type("question_font_desc", ustr));
+    ustr = m_answer_font_desc.to_string ();
     m_config->set_pair ("Default", 
-						IniFile::tuple_type("answer_font_desc",usfont));
-    usfont = m_example_font_desc.to_string ();
+						IniFile::tuple_type("answer_font_desc", ustr));
+    ustr = m_example_font_desc.to_string ();
     m_config->set_pair ("Default", 
-						IniFile::tuple_type("example_font_desc",usfont));
-    usfont = m_input_font_desc.to_string ();
+						IniFile::tuple_type("example_font_desc", ustr));
+    ustr = m_input_font_desc.to_string ();
     m_config->set_pair ("Default", 
-						IniFile::tuple_type("input_font_desc",usfont));
-    usfont = m_app_font_desc.to_string ();
+						IniFile::tuple_type("input_font_desc", ustr));
+    ustr = m_app_font_desc.to_string ();
     m_config->set_pair ("Default", 
-						IniFile::tuple_type("app_font_desc",usfont));
-
+						IniFile::tuple_type("app_font_desc", ustr));
+	/** Text alignment
+	 */
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("front_x_alignment",	
+											m_front_x_alignment));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("front_y_alignment",	
+											m_front_y_alignment));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("front_x_padding",	
+											m_front_x_padding));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("front_y_padding",	
+											m_front_y_padding));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("front_justification",	
+											m_front_justification));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("back_x_alignment",	
+											m_back_x_alignment));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("back_y_alignment",	
+											m_back_y_alignment));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("back_x_padding",	
+											m_back_x_padding));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("back_y_padding",	
+											m_back_y_padding));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("back_justification",	
+											m_back_justification));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("example_x_alignment",	
+											m_example_x_alignment));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("example_y_alignment",	
+											m_example_y_alignment));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("example_x_padding",	
+											m_example_x_padding));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("example_y_padding",	
+											m_example_y_padding));
+    m_config->set_pair ("Default", 
+						IniFile::tuple_type("example_justification",	
+											m_example_justification));
     /** Geometry
      */
     m_config->set_pair ("Default", 
@@ -746,7 +927,7 @@ get_flipside_history (const string& fname_)
 {
     trace_with_mask("GrappConf::get_flipside_history",GUITRACE);
 
-	if (fname_.find ("Untitled") != std::string::npos) {
+	if (fname_.find ("Untitled") != string::npos) {
 		return (FRONT);
 	}
 	flipside_iter_t iter = m_flipside_history.begin ();
@@ -817,7 +998,7 @@ set_proj_name (const string& fname_)
 */
 void
 GrappConf::
-str_to_geometry (const std::string& src_, Gdk::Rectangle& aspect_)
+str_to_geometry (const string& src_, Gdk::Rectangle& aspect_)
 {
     trace_with_mask("GrappConf::str_to_geometry",GUITRACE);
 
@@ -843,3 +1024,4 @@ geometry_to_str (const Gdk::Rectangle& aspect_)
     sprintf (buf, "%dx%d", aspect_.get_height (), aspect_.get_width  ());
     return buf;
 }
+
