@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                               MyPropertyBox.cpp
 //------------------------------------------------------------------------------
-// $Id: MyPropertyBox.cpp,v 1.6 2006/03/19 15:39:44 vlg Exp $
+// $Id: MyPropertyBox.cpp,v 1.7 2006/12/11 03:07:22 vlg Exp $
 //------------------------------------------------------------------------------
 //  Copyright (c) 2002-2004 by Vladislav Grinchenko
 //
@@ -75,8 +75,10 @@ MyPropertyBox (const std::string& title_)
 		mem_fun (*this, &MyPropertyBox::hildon_im_close_cb));
 #endif
 
+	m_notebook->set_show_border ();	// to draw a bevel around the pages
+	m_notebook->set_scrollable ();  // set the tab label area scrollable
+
 	get_vbox ()->pack_start (*m_notebook);
-	m_notebook->set_show_border (true);
 
 	show_all_children ();
 }
