@@ -1,6 +1,6 @@
 // -*- c++ -*-
 //------------------------------------------------------------------------------
-// $Id: PrefWindow.h,v 1.9 2006/03/19 15:39:44 vlg Exp $
+// $Id: PrefWindow.h,v 1.11 2006/09/03 19:26:36 vlg Exp $
 //------------------------------------------------------------------------------
 //                            PrefWindow.h
 //------------------------------------------------------------------------------
@@ -21,12 +21,13 @@
 #include <gtkmm/main.h>
 
 #include "Granule-main.h"
-#include "Config.h"
+#include "GrappConf.h"
 #include "MyPropertyBox.h"
 #include "GeneralPref.h"
 #include "SchedulingPrefs.h"
 #include "SoundPrefs.h"
 #include "GeometryPrefs.h"
+#include "AppearancePrefs.h"
 
 class PrefWindow : public MyPropertyBox
 {
@@ -48,10 +49,11 @@ private:
     void stop ();
 
 private:
-	GeneralPref     m_general_pref;
-	SchedulingPrefs m_scheduling_pref;
-	SoundPrefs      m_sound_pref;
-	GeometryPrefs   m_geometry_pref;
+	GeneralPref      m_general_pref;
+	AppearancePrefs  m_appearance_pref;
+	SchedulingPrefs  m_scheduling_pref;
+	SoundPrefs       m_sound_pref;
+	GeometryPrefs    m_geometry_pref;
 
 	bool m_ignore_callbacks;
 	bool m_changed;
