@@ -2,7 +2,7 @@
 //------------------------------------------------------------------------------
 //                              VCard.h
 //------------------------------------------------------------------------------
-// $Id: VCard.h,v 1.7 2006/09/06 02:07:59 vlg Exp $
+// $Id: VCard.h,v 1.8 2006/12/18 03:33:15 vlg Exp $
 //
 // Date: Apr 24 2004
 //------------------------------------------------------------------------------
@@ -17,6 +17,8 @@ using Glib::ustring;
 
 #include <assa/TimeVal.h>
 
+#include "Granule-main.h"
+
 class VCard 
 {
 public:
@@ -26,12 +28,13 @@ public:
 	virtual void set_question     (const ustring& q_) = 0;
 	virtual void set_answer       (const ustring& a_) = 0;
 	virtual void set_example      (const ustring& e_) = 0;
-	virtual void set_alt_spelling (const ustring& a_) = 0;
+	virtual void set_alt_spelling (SideSelection side_,
+								   const ustring& a_) = 0;
 
 	virtual ustring get_question     () const = 0;
 	virtual ustring get_answer       () const = 0;
 	virtual ustring get_example      () const = 0;
-	virtual ustring get_alt_spelling () const = 0;
+	virtual ustring get_alt_spelling (SideSelection side_) const = 0;
 	virtual long    get_id           () const = 0;
 
 	void set_reversed (bool r_) { m_reversed = r_;   }
